#!/usr/bin/perl 
my $start_run = time();

use Net::DNS;
use Net::IP;
#use strict;
my $rr;

my $res = Net::DNS::Resolver->new();

$url = 'http://www.corpocrat.com';

use URI;
($domain = URI->new("$url")->authority) =~ s/^www\.//i ;

unless($domain) {
die "FATAL: No domain found..\n";
}

my $dest = 'mail2web.com';
my @parent_ns;

my @nservers = ();
my @nsips = ();

$res->defnames(0);
$res->retry(2);



################# NS records from DOMAIN  ###############################

my $ns_req = $res->query($domain, "NS");
die "No nameservers found for $domain: ", $res->errorstring, "\n"
	unless defined($ns_req) and ($ns_req->header->ancount > 0);


# Send out non-recursive queries
$res->recurse(0);
# Do not buffer standard out
$| = 1;


# Check the SOA record on each nameserver.

foreach my $nsrr (grep {$_->type eq "NS" } $ns_req->answer) {
	
	
	my $ns = $nsrr->nsdname;

	push(@nservers,$ns);
	
	my $local_res = Net::DNS::Resolver->new();
	
	my $a_req = $local_res->query($ns, 'A');


	unless ($a_req) {
		warn "A record fail $ns (no glue): ", $res->errorstring, "\n";
		next;
	}
	
	foreach my $ip (map { $_->address } grep { $_->type eq 'A' } $a_req->answer) {
		
		$res->nameservers($ip);	
	
		print "$ns ($ip)";
		push (@nsips,$ip);

 	my $soa_req = $res->send($domain, 'SOA', 'IN');
		
		unless (defined($soa_req)) {
			warn "SOA fail for $ns:",$res->errorstring, "\n";
			next;
		}

		#----------------------------------------------------------------------
		# Is this nameserver authoritative for the domain?
		#----------------------------------------------------------------------

		unless ($soa_req->header->aa) {
			warn "Nameserver NOT authoritative or LAME delegation \n";
			next;
		} else 	{
                print "[AUTHORITATIVE]\n";
		}


		
	}
}
unless (@nservers) {
die "FAIL: Unable to find nameservers....\n";
}

######## Checking A in NS records ####################

print" Checking IP addresses in NS recordss..\n";


@ns_out = isIP(@nservers);

unless(@ns_out) {
print "OK. All NS records are hostnames which cannot have IPs\n";
} else {
print "Error - IPs found in NS records instead of nameserver names..\n";
}

############ Checking NS for CNAME #######################

print "Checking NS for CNAMEs....\n";

foreach $nsc (@nservers) {
	$nsc_query = $res->query($nsc,"CNAME");
	if($nsc_query) {
 	  foreach my $ns_cname ($nsc_query->answer) {
           $ns_cname->print, "\n";
   	   }
	  } else {
      warn "OK. CNAME not found with NS records ", $res->errorstring, "\n";
	}
  }


################### Checking CNAME in domain ################

# CNAME along with NS may confuse nameservers

print "Checking CNAME for $domain for NS...\n";
my $cns_query = $res->query($domain,"CNAME");

if ($cns_query) {
   foreach my $cns ($cns_query->answer) {
          print $cns->print, "\n";
      }
  } else {
      warn "OK. CNAME not found with NS records ", $res->errorstring, "\n";
  }


########################### PARENT NAMESERVERS ##############

use Net::DNS::Resolver::Recurse;
my $pres = Net::DNS::Resolver::Recurse->new;
$pres->tcp_timeout(2);
$pres->udp_timeout(2);
$pres->debug(0);

my @root_ns = map $_ . '.root-servers.net', 'a'..'m';
$pres->hints(@root_ns);

my $packet = $pres->query_dorecursion($domain, "NS");
print "Parent Nameservers:\n";

foreach my $pns ($packet->additional)
{
   print $pns->name,"(", $pns->rdatastr,")\n";  # ip
    push @parent_ns,$pns->name;
}
############### STEALTH NAMESERVERS ########################

 @union = @intersection = @difference = ();
    %count = ();
    foreach $element (@nservers, @parent_ns) { $count{$element}++ }
    foreach $element (keys %count) {
        push @union, $element;
        push @{ $count{$element} > 1 ? \@intersection : \@difference }, $element;
    }
print "Stealth nameservers:\n";
if(@difference) {
 foreach $out(@difference) {
 print $out,"\n";
 }
}else {
print "OK. none found\n";
}

################# Nameserver IPs on same SUBNET ###################

print "Checking Nameserver IPs on different subnets...\n";
use NetAddr::IP;
my $in_subnet = 1;

foreach $ip1 (@nsips) {

my $first  = NetAddr::IP->new($ip1.'/24');

foreach $ip2 (@nsips) {
my $second = NetAddr::IP->new($ip2.'/32');

if ($ip1 eq $ip2 ) { next; }

if ($second->within($first)) {
    #printf "%s is within %s\n", $second, $first;
} else {
    #printf "%s is not within %s\n", $second, $first;
    $in_subnet = 0;
   }
 }
}

if ($in_subnet eq 1) {
print " All IPs are in SAME subnet\n";
} else  {
print "OK. 1 or more nameservers are outside of same subnet.\n";
}


########################### SOA RECORD ################
print "SOA record\n";

                my $soa_query = $res->query($domain, 'SOA');

                if ($soa_query) {
		($soa_query->answer)[0]->print, "\n";
		}
		else  {
                 print "SOA record fail. Result:\n",$res->errorstring;
                }
if ($soa_query) {               
print "Serial: ",($soa_query->answer)[0]->serial,"\n";
print "Refresh: ",($soa_query->answer)[0]->refresh,"\n";
print "Retry: ",($soa_query->answer)[0]->retry,"\n";
print "Expire: ",($soa_query->answer)[0]->expire,"\n";
print "Minimum TTL: ",($soa_query->answer)[0]->minimum,"\n";

}

  

########### WEBSITE A RECORD  ###################

  my $a_query = $res->search($domain);
  
print "Website IP Address (A Record)...\n";

  if ($a_query) {
      foreach my $rr ($a_query->answer) {
          next unless $rr->type eq "A";
          print $rr->address, "\n";
      }
  } else {
      warn "Unable to obtain A record for your website: ", $res->errorstring, "\n";
  }

################ CNAME for WWW #####################

print "Checking WWW entry in CNAME for $domain \n";

$www_domain = "www.".$domain;

my $cname_query = $res->query($www_domain,"CNAME");

if ($cname_query) {
   foreach my $cnrr ($cname_query->answer) {
           $cnrr->print, "\n";
      }
  } else {
      warn "CNAME entry for WWW not found: ", $res->errorstring, "\n";
  }

############# Checking A record for WWW ##################

print "Checking WWW entry with A record \n";

my $wa_res = Net::DNS::Resolver->new();
my $wa_query = $wa_res->search($www_domain);


  if ($wa_query) {
      foreach my $wa ($wa_query->answer) {
          print $wa->string, "\n";
      }
  } else {
      warn "Unable to retrieve A for WWW entry: ", $res->errorstring, "\n";
  }


#############  RECURSION CHECK #######################

print "Checking recursion (open dns server)...\n";

  my $rres = Net::DNS::Resolver->new(
        nameservers => [@nservers],
        recurse     => 1,       debug       => 0,
  );
$rres->tcp_timeout(3);
$rres->udp_timeout(3);

my $r_query = $rres->query($dest,"A");
my @dest_ips=();

  if ($r_query) {
      foreach my $rrr ($r_query->answer) {
          next unless $rrr->type eq "A";
          print "Recursive lookups are ENABLED in ",$res->nameserver(),": ",$rrr->address, "\n";
	 # push(@dest_ips,$rrr->address);
      }
  } else {
      warn "OK. Recursive queries are not answered.  Result: ", $rres->errorstring, "\n";
  }

############ MX RECORD CHECK #####################

print "\n MX mail exchanger records....\n";

my $mx_res = Net::DNS::Resolver->new();
#$mx_res->tcp_timeout(3);
my $mx_req = $mx_res->query($domain, 'MX');

if($mx_req) {
foreach my $mrr (grep {$_->type eq "MX" } $mx_req->answer) {

	my $ms = $mrr->exchange;
        push (@mx_servers,$ms);


        unless ($mx_req) {
                warn "MX record fail: ", $mx_res->errorstring, "\n";
                next;
        }

        my $ma_req = $mx_res->query($ms, 'A');


        foreach my $mip (map { $_->address } grep { $_->type eq 'A' } $ma_req->answer) {

              if ($mip) { 
		 print "$ms ($mip)\n ";
                push (@mx_ips,$mip);
		} else {
		 print "$ms (IP FAIL)\n";
		}	

      }
 }
################## Checking MX for pointing to CNAME #####################
 
print "Checking MX hosts pointing to CNAME...\n";

if (@mx_servers) {
	foreach $mxe (@mx_servers) {
        $mc_query = $res->query($mxe,"CNAME");

		
	if ($mc_query) {
   	foreach my $mcrr ($mc_query->answer) {
          print $mcrr->print, "\n";
      		}
  		} else {
  	    warn "OK. No CNAME found for MX hosts: ", $res->errorstring, "\n";
 		 }

	}

}


######################### Checking MX are hosts not IPs #####################

print "Checking MX are hosts and not IPs...\n";


@out = isIP(@mx_servers);

unless(@out) {
print "OK. All MX records are hostnames";
} else {
print "Error - IPs found in MX records..";
}

sub isIP(@servers) {

foreach $ms (@mx_servers) {

 if ($ms =~ /(\d+)(\.\d+){3}/) {
      # print "$ms is IP\n";
     push(@result,$ms);
  }
 }
return @result;
}

####################### Reverse MX Records ########################

print "\nChecking Reverse MX Records ...\n";

#my $mx_res = Net::DNS::Resolver->new();

foreach $mxip(@mx_ips) {
$ip = new Net::IP ($mxip) or die (Net::IP::Error());
my $mxanswer = $mx_res->query($ip->reverse_ip(),'PTR');

if($mxanswer) {

 foreach my $mrr (grep {$_->type eq "PTR" } $mxanswer->answer) {
  print $ip->reverse_ip()," PTR ",$mrr->ptrdname,"\n";
  }
  } else {
  print "No Reverse IP \n";
  }

}
} else {
  print "MX records not found\n"; 
}


############ SPF RECORD #################################

my $spf_query = $res->query($domain,"TXT");
  if ($spf_query) {

      foreach $rr ($spf_query->answer) {
	#next unless $rr->txtdata eq '/v=spf/';
          print "SPF Record Installed OK: ",$rr->txtdata, "\n";
      }
  }
  else {
      warn "Unable to get SPF Record : ", $res->errorstring, "\n";
  }

############## DOMAIN KEYS ################################

my $dk_query = $res->query("_domainkey.".$domain,"TXT");
  if ($dk_query) {

      foreach $rr ($dk_query->answer) {
          print "Domain Keys installed OK: ",$rr->print,$rr->txtdata, "\n";
      }
  }
  else {
      warn "Unable to obtain Domain Keys: ", $res->errorstring, "\n";
  }



########### ZONE TRANSFER AXFR CHECK ####################
my $zres = Net::DNS::Resolver->new();
 $zres->tcp_timeout(5);
    $zres->nameservers($nservers[0]);
    my @zone = $zres->axfr($domain);


    if (@zone) {
    print "ALERT: Zone transfer AXFR is ENABLED.\n";
    # foreach my $rr (@zone) {
    #        $rr->print;
    #    }
    } else {
        print 'OK. Zone transfer failed: ', $zres->errorstring, "\n";
    }

################## DNS Version #########################

my $end_run = time();
my $run_time = $end_run - $start_run;
print "Job took $run_time seconds\n";

